#
# Black and White CCS file Body-Partanalyzer V0.1
#   	V0.1
#
# This script is meant to export all of the body parts as seperate
# OBJ files, as described  in the CCS file. The file name corresponds
# to the body-part number given to the collection of vertices in the
# file. In doing this, the body part described by each number can
# be determined
#    There are two options. The body-part groups are defined twice in
# the CCS file. Once, each vertex is assigned to its 'main' body part.
# In this defenition, each vertex is assigned to only one body part.
# The second definition describes which faces are assigned to each
# body part. In this definition, each vertex (making up the indicated
# faces) can be part of up to three body parts. Three is a limitation
# set by the game's data structure (bone-weight definition in the CSK
# file).
#    There is also the choice of exporting only the vertices, not the
# faces. Some programs like blender will not show vertices not attached
# to a face if there are any faces at all (by default). There are
# body parts that have vertices not assigned to a complete face.
#
# 
#
# A folder named as the creature file provided is created. Each body part
# is then extracted into OBJ files in this folder. 
#
#
# This script is less 'clean' than other scripts.
#
#



import struct
import sys
import os




################################################
#           Variables Used                     #
################################################
#unused - CCS_IK_ARRAY_NUM = 0
#unused - CCS_IK_ARRAY_ENTRY_NUM = 0
CCS_VERTEX_COUNT = 0
CCS_FACE_COUNT = 0
CCS_BODY_PART_COUNT = 0


#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0


#Hold entire contents of CSK in memory
CCSFilebuffer = ''
#Holds a current address into the CCSFilebuffer used for reading
#variable length portion of the file
CCSFilebufferPT = 0



#scratch space
scratchvalue = ''
#holds temporary values
valuebuffer = ''



#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''


#Option related Vars
#-------------------
userInput =''
#Which body-part set to export. 0 = main associations,
#1 = weight associations
BPset = 0
#Export the faces or not
FaceExp = 1


#Strings collect values to write to the OBJ files
vertString = ''
normalString = ''
faceString = ''


#unused - CCSIKChains = []
CCSVerts = []
CCSVertPartArray = []
CCSNormals = []
CCSFaces = []
CCSBoundingBox = []
CCSFaceGroupArrays = []


#Keeps track of which verticies have been found to belong
#to a body part while generating each part. Used to correctly
#generate the faces for that body part
vertexUsed = []


#Keeps track of which FACES have been found to belong
#to a body part while generating each part. Used to correctly
#generate the faces for that body part
faceUsed = []


###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  A CCS file"
	print
	raise SystemExit

filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################




################################################
#                 Options                      #
################################################
print
print
print
print "Export which body part set?"
print "  1) Vertex-part association"
print "  2) Face-part association"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "2":
	BPset = 1
userInput = ''


print
print
print
print "Export Faces?"
print "  1) Yes"
print "  2) No"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "2":
	FaceExp = 0
userInput = ''
################################################
################################################




###########################################
#            open CCS                     #
###########################################
try:
	orfile = open(pathtomodelnoext+".ccs", 'rb')
except:
	print
	print "ERROR:Cannot open CCS file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit
################################################
################################################



###########################################
#            create folder                #
###########################################
if not os.path.isdir(filenamenoext):
	try:
		os.makedirs(filenamenoext)
	except:
		print
		print "Cannot create output folder"
		print "in the current working directory"
		print "Permission problems?"
		print
		orfile.close()
		raise SystemExit
################################################
################################################








################################################
#           Skip CCS header and IK arrays      #
################################################

orfile.seek(0)
CCSFilebuffer = orfile.read()
CCSFilebufferPT = 0
scratchvalue = ''

CCSFilebufferPT += 44

###This is the number of entries in the IK arrays. each
###entry is a DWORD = 4 bytes
valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
scratchvalue = struct.unpack('<i', valueBuffer)[0]

CCSFilebufferPT += ( scratchvalue * 4)
scratchvalue = ''
################################################
################################################





################################################
#           Read CCS vertex array              #
################################################
valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_VERTEX_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_VERTEX_COUNT) + "\n"



i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSVerts.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1

################################################
################################################




################################################
#    Read CCS Vertex-to-part index array       #
################################################

i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	CCSVertPartArray.append(struct.unpack('<i', valueBuffer)[0])
	i += 1
################################################
################################################




################################################
#      Read CCS Normal array                   #
################################################

i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSNormals.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1
################################################
################################################




################################################
#      Read Face Info array                    #
################################################

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_FACE_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_FACE_COUNT) + "\n"


################################################
#      Skip eye definition section             #
################################################
CCSFilebufferPT += (CCS_FACE_COUNT * 4)


i = 0 
while i < CCS_FACE_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSFaces.append( ( struct.unpack('<3i', valueBuffer)[0], struct.unpack('<3i', valueBuffer)[1], struct.unpack('<3i', valueBuffer)[2] ) )
	i += 1

################################################
################################################



################################################
#        Read bounding box Array               #
################################################
valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_BODY_PART_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_BODY_PART_COUNT) + "\n\n"


i = 0
while i < CCS_BODY_PART_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+24]
	CCSFilebufferPT += 24
	CCSBoundingBox.append( ( struct.unpack('<6f', valueBuffer)[0], struct.unpack('<6f', valueBuffer)[1], struct.unpack('<6f', valueBuffer)[2], struct.unpack('<6f', valueBuffer)[3], struct.unpack('<6f', valueBuffer)[4], struct.unpack('<6f', valueBuffer)[5] ) )
	i += 1

################################################
################################################



################################################
#        Read face group array                 #
################################################

i = 0
while i < CCS_BODY_PART_COUNT:
	CCSFaceGroupArrays.append([])
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	j = struct.unpack('<i', valueBuffer)[0]
	CCSFaceGroupArrays[i].append(j)
	k = 0
	while k < j:
			valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
			CCSFilebufferPT += 4
			CCSFaceGroupArrays[i].append( struct.unpack('<i', valueBuffer)[0] )
			k += 1
	i += 1


################################################
################################################







################################################
#        Create BodyPart files                 #
################################################

###intialize vertex used array
i = 0
while i < CCS_VERTEX_COUNT:
	vertexUsed.append([])
	i += 1


################################################
#       Using vertex-part association          #
################################################
if BPset == 0:
	i = 0
	while i < CCS_BODY_PART_COUNT:
		try:
			owfile = open(filedir+filenamenoext+"\\"+filenamenoext+"-"+str(i)+".obj", 'w')
		except:
			print
			print "ERROR: Cannot open text file for writing"
			print "in the folder"
			print str(filedir)+filenamenoext
			print "Persmission problems?"
			print
			orfile.close()
			raise SystemExit

		###initialize vertex used array
		j = 0
		while j < CCS_VERTEX_COUNT:
			vertexUsed[j] = 0
			j += 1

		j = 0
		while j < CCS_VERTEX_COUNT:
			###If this vertex (j) is in this body part (i)
			if CCSVertPartArray[j] == i:
				vertString += "v " + str(CCSVerts[j][0]) + " " + str(CCSVerts[j][1]) + " " + str(CCSVerts[j][2]) + "\n"
				normalString += "vn " + str(CCSNormals[j][0]) + " " + str(CCSNormals[j][1]) + " " + str(CCSNormals[j][2]) + "\n"
				###Vertex is part of this part
				vertexUsed[j] = 1
			j += 1

		###Now, since the vertices are being assigned different numbers, the indexes used in the face array
		###will be wrong for the new OBJ file.. create a mapping to the new index number in the
		###vertex used array. ALSO!! Remember that OBJ indexes start at "1", not 0, so +1
		### (k = 1 to start, instead of k = 0)
		j = 0
		k = 1
		while j < CCS_VERTEX_COUNT:
			if vertexUsed[j] == 1:
				vertexUsed[j] = k
				k += 1
			j += 1

		###Create faces. Each face in the face array for which all three vertices are
		###in this part will be added to this part's OBJ file
		j = 0
		while j < CCS_FACE_COUNT:
			if (vertexUsed[CCSFaces[j][0]]!=0) and (vertexUsed[CCSFaces[j][1]]!=0) and (vertexUsed[CCSFaces[j][2]]!=0):
				faceString += "f " + str(vertexUsed[CCSFaces[j][0]]) + "//" + str(vertexUsed[CCSFaces[j][0]]) + " " + str(vertexUsed[CCSFaces[j][1]]) + "//" + str(vertexUsed[CCSFaces[j][1]]) + " " + str(vertexUsed[CCSFaces[j][2]]) + "//" + str(vertexUsed[CCSFaces[j][2]]) + "\n"
			j += 1

		owfile.write(vertString)
		owfile.write(normalString)
		if FaceExp:
			owfile.write(faceString)
		owfile.close()

		vertString = ''
		normalString = ''
		faceString = ''

		i += 1


################################################
#       Using face-part association            #
################################################
faceused = 0

if BPset == 1:
	i = 0
	while i < CCS_BODY_PART_COUNT:
		try:
			owfile = open(filedir+filenamenoext+"\\"+filenamenoext+"-"+str(i)+".obj", 'w')
		except:
			print
			print "ERROR: Cannot open text file for writing"
			print "in the folder"
			print str(filedir)+filenamenoext
			print "Persmission problems?"
			print
			orfile.close()
			raise SystemExit

		###initialize vertex used array
		j = 0
		while j < CCS_VERTEX_COUNT:
			vertexUsed[j] = 0
			j += 1

		###The number of faces in this array
		j = CCSFaceGroupArrays[i][0]
		###First value in the part array is the count, start at the second value
		k = 1
		while k < j:
			###For each face, mark the vertices that make up that face as 'used'
			faceused = CCSFaceGroupArrays[i][k]
			vertexUsed[CCSFaces[faceused][0]] = 1
			vertexUsed[CCSFaces[faceused][1]] = 1
			vertexUsed[CCSFaces[faceused][2]] = 1
			k += 1



		###Now, since the vertices are being assigned different numbers, the indexes used in the face array
		###will be wrong for the new OBJ file.. create a mapping to the new index number in the
		###vertex used array. ALSO!! Remember that OBJ indexes start at "1", not 0, so +1
		### (k = 1 to start, instead of k = 0)
		j = 0
		k = 1
		while j < CCS_VERTEX_COUNT:
			if vertexUsed[j] == 1:
				vertexUsed[j] = k
				vertString += "v " + str(CCSVerts[j][0]) + " " + str(CCSVerts[j][1]) + " " + str(CCSVerts[j][2]) + "\n"
				normalString += "vn " + str(CCSNormals[j][0]) + " " + str(CCSNormals[j][1]) + " " + str(CCSNormals[j][2]) + "\n"
				k += 1
			j += 1

		###Create faces. Each face in the face array for which all three vertices are
		###in this part will be added to this part's OBJ file
		j = 0
		while j < CCS_FACE_COUNT:
			if (vertexUsed[CCSFaces[j][0]]!=0) and (vertexUsed[CCSFaces[j][1]]!=0) and (vertexUsed[CCSFaces[j][2]]!=0):
				faceString += "f " + str(vertexUsed[CCSFaces[j][0]]) + "//" + str(vertexUsed[CCSFaces[j][0]]) + " " + str(vertexUsed[CCSFaces[j][1]]) + "//" + str(vertexUsed[CCSFaces[j][1]]) + " " + str(vertexUsed[CCSFaces[j][2]]) + "//" + str(vertexUsed[CCSFaces[j][2]]) + "\n"
			j += 1

		owfile.write(vertString)
		owfile.write(normalString)
		if FaceExp:
			owfile.write(faceString)
		owfile.close()

		vertString = ''
		normalString = ''
		faceString = ''

		i += 1



################################################
################################################


#Close CCS
orfile.close()



