#
# Black and White CCS file Bounding Box Analyzer V0.1
#   	V0.1
#
# This script is mean to export the bounding boxes. It uses the
# CCS file data analyzer script and simply skips the unneeded outputs.
# Each body part is exported, centered around the origin. Each
# bounding box is exported as an OBJ files named with the body part
# number and "_BB". For example, "47_BB.obj"
#
# A folder named as the creature file provided is created. Each
# Bounding Boc is then extracted into the folder
#
# It looks like the idea behind this test is not correct. The body part
# is not 'centered around the origin'. Rather, it appears that it is
# transformed into position by a transformation matrix, probably found
# in the CBN file.
#
#
#



import struct
import sys
import os




################################################
#           Variables Used                     #
################################################
CCS_IK_ARRAY_NUM = 0
CCS_IK_ARRAY_ENTRY_NUM = 0
CCS_VERTEX_COUNT = 0
CCS_FACE_COUNT = 0
CCS_BODY_PART_COUNT = 0


#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0


#Hold entire contents of CCS in memory
CCSFilebuffer = ''
#Holds a current address into the CCSFilebuffer used for reading
#variable length portion of the file
CCSFilebufferPT = 0



#scratch space
scratchvalue = ''
#holds temporary values
valuebuffer = ''



#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''



#Option related Vars
#-------------------
userInput =''
#Export the faces or not
FaceExp = 1



CCSIKChains = []
CCSVerts = []
CCSVertPartArray = []
CCSNormals = []
CCSFaces = []
CCSBoundingBox = []
CCSFaceGroupArrays = []




###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  A CCS file"
	print
	raise SystemExit

filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################




################################################
#                 Options                      #
################################################

print
print
print
print "Export body part Faces?"
print "  1) Yes"
print "  2) No"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "2":
	FaceExp = 0
userInput = ''
################################################
################################################




###########################################
#            open CCS                     #
###########################################
try:
	orfile = open(pathtomodelnoext+".ccs", 'rb')
except:
	print
	print "ERROR:Cannot open CCS file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit
################################################
################################################



###########################################
#            create folder                #
###########################################
if not os.path.isdir(filenamenoext):
	try:
		os.makedirs(filenamenoext)
	except:
		print
		print "Cannot create output folder"
		print "in the current working directory"
		print "Permission problems?"
		print
		orfile.close()
		raise SystemExit
################################################
################################################








################################################
#           Read CCS header                    #
################################################

orfile.seek(0)
CCSFilebuffer = orfile.read()
CCSFilebufferPT = 0
scratchvalue = ''

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+40]
CCSFilebufferPT += 40
#scratchvalue += valueBuffer + "\n"

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
#scratchvalue += str(struct.unpack('<i', valueBuffer)[0]) + "\n"


################################################
################################################



################################################
#           Create CCS IK file                 #
################################################

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
#scratchvalue += str( struct.unpack('<i', valueBuffer)[0] ) + "\n"


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_IK_ARRAY_NUM = struct.unpack('<i', valueBuffer)[0]
#scratchvalue += str(CCS_IK_ARRAY_NUM) + "\n"


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_IK_ARRAY_ENTRY_NUM = struct.unpack('<i', valueBuffer)[0]
#scratchvalue += str(CCS_IK_ARRAY_ENTRY_NUM) + "\n"

i = 0
while i < CCS_IK_ARRAY_NUM:
	j = 0
	CCSIKChains.append([])
	while j < CCS_IK_ARRAY_ENTRY_NUM:
		valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
		CCSFilebufferPT += 4
		CCSIKChains[i].append(struct.unpack('<i', valueBuffer)[0])
		j += 1
	i += 1



################################################
################################################



################################################
#           Create CCS vertex file             #
################################################

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_VERTEX_COUNT = struct.unpack('<i', valueBuffer)[0]
#scratchvalue += str(CCS_VERTEX_COUNT) + "\n"




i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSVerts.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1



################################################
################################################




################################################
#  Create CCS Vertex-to-part index file        #
################################################



i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	CCSVertPartArray.append(struct.unpack('<i', valueBuffer)[0])
	i += 1


################################################
################################################


################################################
#      Create CCS Normal array file            #
################################################



i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSNormals.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1


################################################
################################################


################################################
#      Create Face Info file                   #
################################################


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_FACE_COUNT = struct.unpack('<i', valueBuffer)[0]
#scratchvalue += str(CCS_FACE_COUNT) + "\n"

################################################
################################################


################################################
#      Skip eye definition section             #
################################################
CCSFilebufferPT += (CCS_FACE_COUNT * 4)



################################################
#            Create Face file                  #
################################################


i = 0 
while i < CCS_FACE_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSFaces.append( ( struct.unpack('<3i', valueBuffer)[0], struct.unpack('<3i', valueBuffer)[1], struct.unpack('<3i', valueBuffer)[2] ) )
	i += 1



################################################
################################################



################################################
#        Create bounding box files             #
################################################
curX1 = 0.0
curY1 = 0.0
curZ1 = 0.0
curX2 = 0.0
curY2 = 0.0
curZ2 = 0.0


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_BODY_PART_COUNT = struct.unpack('<i', valueBuffer)[0]
#scratchvalue += str(CCS_BODY_PART_COUNT) + "\n\n"


i = 0
while i < CCS_BODY_PART_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+24]
	CCSFilebufferPT += 24
	CCSBoundingBox.append( ( struct.unpack('<6f', valueBuffer)[0], struct.unpack('<6f', valueBuffer)[1], struct.unpack('<6f', valueBuffer)[2], struct.unpack('<6f', valueBuffer)[3], struct.unpack('<6f', valueBuffer)[4], struct.unpack('<6f', valueBuffer)[5] ) )
	i += 1

i = 0
while i < CCS_BODY_PART_COUNT:
	try:
		owfile = open(filedir+filenamenoext+"\\"+str(i)+"_BB.obj", 'w')
	except:
		print
		print "ERROR: Cannot open text file for writing"
		print "in the folder"
		print str(filedir)+filenamenoext
		print "Permission problems?"
		print
		orfile.close()
		raise SystemExit

	scratchvalue = ''

#	scratchvalue += "v " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][2]) + "\n"

###Swap X,Y values
#	scratchvalue += "v " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][5]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][2]) + "\n"
#	scratchvalue += "v " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][2]) + "\n"

###Running Value idea - Also uses swapped X,Y values
	curX1 += CCSBoundingBox[i][1]
	curY1 += CCSBoundingBox[i][0]
	curZ1 += CCSBoundingBox[i][2]
	curX2 = curX1 + CCSBoundingBox[i][4]
	curY2 = curY1 + CCSBoundingBox[i][3]
	curZ2 = curZ1 + CCSBoundingBox[i][5]

	scratchvalue += "v " + str(curX1) + " " + str(curY1) + " " + str(curZ1) + "\n"
	scratchvalue += "v " + str(curX1) + " " + str(curY2) + " " + str(curZ1) + "\n"
	scratchvalue += "v " + str(curX1) + " " + str(curY1) + " " + str(curZ2) + "\n"
	scratchvalue += "v " + str(curX1) + " " + str(curY2) + " " + str(curZ2) + "\n"

	scratchvalue += "v " + str(curX2) + " " + str(curY2) + " " + str(curZ2) + "\n"
	scratchvalue += "v " + str(curX2) + " " + str(curY1) + " " + str(curZ2) + "\n"
	scratchvalue += "v " + str(curX2) + " " + str(curY2) + " " + str(curZ1) + "\n"
	scratchvalue += "v " + str(curX2) + " " + str(curY1) + " " + str(curZ1) + "\n"
#######################


	scratchvalue += "f 1 2" + "\n"
	scratchvalue += "f 1 3" + "\n"
	scratchvalue += "f 2 4" + "\n"
	scratchvalue += "f 3 4" + "\n"
	scratchvalue += "f 5 6" + "\n"
	scratchvalue += "f 5 7" + "\n"
	scratchvalue += "f 6 8" + "\n"
	scratchvalue += "f 7 8" + "\n"

	scratchvalue += "f 1 8" + "\n"
	scratchvalue += "f 2 7" + "\n"
	scratchvalue += "f 3 6" + "\n"
	scratchvalue += "f 4 5" + "\n"



	owfile.write(scratchvalue)
	owfile.close()

	i += 1


################################################
################################################

################################################
#        Read face group array                 #
################################################

i = 0
while i < CCS_BODY_PART_COUNT:
	CCSFaceGroupArrays.append([])
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	j = struct.unpack('<i', valueBuffer)[0]
	CCSFaceGroupArrays[i].append(j)
	k = 0
	while k < j:
			valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
			CCSFilebufferPT += 4
			CCSFaceGroupArrays[i].append( struct.unpack('<i', valueBuffer)[0] )
			k += 1
	i += 1


################################################
################################################



################################################
#        Write out body parts                  #
################################################
vertString = ''
normalString = ''
faceString = ''
vertexUsed = []
partFound = 0
maxX = 0
maxY = 0
maxZ = 0
minX = 0
minY = 0
minZ = 0
#"disp" = displacement
dispX = 0
dispY = 0
dispZ = 0


###initialize vertex used array
j = 0
while j < CCS_VERTEX_COUNT:
	vertexUsed.append( int(0) )
	j += 1

i = 0
while i < CCS_BODY_PART_COUNT:
	try:
		owfile = open(filedir+filenamenoext+"\\"+filenamenoext+"-"+str(i)+".obj", 'w')
	except:
		print
		print "ERROR: Cannot open text file for writing"
		print "in the folder"
		print str(filedir)+filenamenoext
		print "Persmission problems?"
		print
		orfile.close()
		raise SystemExit

	###Reset vertex used array
	j = 0
	while j < CCS_VERTEX_COUNT:
		vertexUsed[j] = 0
		j += 1


	###initialize min/max vertex values. Set them to the first vertex
	###found for the body part
	partFound = 0
	j = 0
	while (j < CCS_VERTEX_COUNT) and (not partFound):
		###If this vertex (j) is in this body part (i)
		if CCSVertPartArray[j] == i:
			maxX = CCSVerts[j][0]
			maxY = CCSVerts[j][1]
			maxZ = CCSVerts[j][2]
			minX = CCSVerts[j][0]
			minY = CCSVerts[j][1]
			minZ = CCSVerts[j][2]
			partFound = 1
		j += 1

	###Find the minimum and maximum values for each vertex
	###J can just continue on from the last value
	while j < CCS_VERTEX_COUNT:
		###If this vertex (j) is in this body part (i)
		if CCSVertPartArray[j] == i:
			if maxX < CCSVerts[j][0]:
				maxX = CCSVerts[j][0]
			if maxY < CCSVerts[j][1]:
				maxY = CCSVerts[j][1]
			if maxZ < CCSVerts[j][2]:
				maxZ = CCSVerts[j][2]
			if minX > CCSVerts[j][0]:
				minX = CCSVerts[j][0]
			if minY > CCSVerts[j][1]:
				minY = CCSVerts[j][1]
			if minZ > CCSVerts[j][2]:
				minZ = CCSVerts[j][2]
		j += 1

	dispX = minX + ((maxX - minX)/2)
	dispY = minY + ((maxY - minY)/2)
	dispZ = minZ + ((maxZ - minZ)/2)


	###Now, write out the OBJ.
	j = 0
	while j < CCS_VERTEX_COUNT:
		###If this vertex (j) is in this body part (i)
		if CCSVertPartArray[j] == i:
			vertString += "v " + str(CCSVerts[j][0] - dispX) + " " + str(CCSVerts[j][1]-dispY) + " " + str(CCSVerts[j][2]-dispZ) + "\n"
			normalString += "vn " + str(CCSNormals[j][0]) + " " + str(CCSNormals[j][1]) + " " + str(CCSNormals[j][2]) + "\n"
			###Vertex is part of this part
			vertexUsed[j] = 1
		j += 1

	###Now, since the vertices are being assigned different numbers, the indexes used in the face array
	###will be wrong for the new OBJ file.. create a mapping to the new index number in the
	###vertex used array. ALSO!! Remember that OBJ indexes start at "1", not 0, so +1
	### (k = 1 to start, instead of k = 0)
	j = 0
	k = 1
	while j < CCS_VERTEX_COUNT:
		if vertexUsed[j] == 1:
			vertexUsed[j] = k
			k += 1
		j += 1

	###Create faces. Each face in the face array for which all three vertices are
	###in this part will be added to this part's OBJ file
	j = 0
	while j < CCS_FACE_COUNT:
		if (vertexUsed[CCSFaces[j][0]]!=0) and (vertexUsed[CCSFaces[j][1]]!=0) and (vertexUsed[CCSFaces[j][2]]!=0):
			faceString += "f " + str(vertexUsed[CCSFaces[j][0]]) + "//" + str(vertexUsed[CCSFaces[j][0]]) + " " + str(vertexUsed[CCSFaces[j][1]]) + "//" + str(vertexUsed[CCSFaces[j][1]]) + " " + str(vertexUsed[CCSFaces[j][2]]) + "//" + str(vertexUsed[CCSFaces[j][2]]) + "\n"
		j += 1

	owfile.write(vertString)
	owfile.write(normalString)
	if FaceExp:
		owfile.write(faceString)
	owfile.close()

	vertString = ''
	normalString = ''
	faceString = ''

	i += 1


#Close CCS
orfile.close()



