#
# Black and White CCS file analyzer V0.1
#   	V0.1
#
# This script is mean to export all of the data structures from a creature
# CCS file as text files for analysis. 
#
# A folder named as the creature file provided is created. Each data
# structure is then extracted into text files in this folder. For a
# description of what the data are, see the CSK outline document.
#
#
# This script is less 'clean' than other scripts.
#
# The 'eye' definition section is skipped
#



import struct
import sys
import os




################################################
#           Variables Used                     #
################################################
CCS_IK_ARRAY_NUM = 0
CCS_IK_ARRAY_ENTRY_NUM = 0
CCS_VERTEX_COUNT = 0
CCS_FACE_COUNT = 0
CCS_BODY_PART_COUNT = 0


#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0


#Hold entire contents of CCS in memory
CCSFilebuffer = ''
#Holds a current address into the CCSFilebuffer used for reading
#variable length portion of the file
CCSFilebufferPT = 0



#scratch space
scratchvalue = ''
#holds temporary values
valuebuffer = ''



#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''






CCSIKChains = []
CCSVerts = []
CCSVertPartArray = []
CCSNormals = []
CCSFaces = []
CCSBoundingBox = []
CCSFaceGroupArrays = []




###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  A CCS file"
	print
	raise SystemExit

filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################





###########################################
#            open CCS                     #
###########################################
try:
	orfile = open(pathtomodelnoext+".ccs", 'rb')
except:
	print
	print "ERROR:Cannot open CCS file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit
################################################
################################################



###########################################
#            create folder                #
###########################################
if not os.path.isdir(filenamenoext):
	try:
		os.makedirs(filenamenoext)
	except:
		print
		print "Cannot create output folder"
		print "in the current working directory"
		print "Permission problems?"
		print
		orfile.close()
		raise SystemExit
################################################
################################################








################################################
#           Read CCS header                    #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\Header.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


orfile.seek(0)
CCSFilebuffer = orfile.read()
CCSFilebufferPT = 0
scratchvalue = ''

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+40]
CCSFilebufferPT += 40
scratchvalue += valueBuffer + "\n"

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
scratchvalue += str(struct.unpack('<i', valueBuffer)[0]) + "\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#           Create CCS IK file                 #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\IK.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
scratchvalue += str( struct.unpack('<i', valueBuffer)[0] ) + "\n"


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_IK_ARRAY_NUM = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_IK_ARRAY_NUM) + "\n"


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_IK_ARRAY_ENTRY_NUM = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_IK_ARRAY_ENTRY_NUM) + "\n"

i = 0
while i < CCS_IK_ARRAY_NUM:
	j = 0
	CCSIKChains.append([])
	while j < CCS_IK_ARRAY_ENTRY_NUM:
		valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
		CCSFilebufferPT += 4
		CCSIKChains[i].append(struct.unpack('<i', valueBuffer)[0])
		j += 1
	i += 1


i = 0
while i < CCS_IK_ARRAY_NUM:
	j = 0
	while j < CCS_IK_ARRAY_ENTRY_NUM:
		scratchvalue += str(CCSIKChains[i][j]) + " "
		j += 1
	scratchvalue += "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#           Create CCS vertex file             #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\vertexInfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit

valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_VERTEX_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_VERTEX_COUNT) + "\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()


try:
	owfile = open(filedir+filenamenoext+"\\vertices.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSVerts.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1

i = 0
while i < CCS_VERTEX_COUNT:
	scratchvalue += str(CCSVerts[i][0]) + " " + str(CCSVerts[i][1]) + " " + str(CCSVerts[i][2]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################




################################################
#  Create CCS Vertex-to-part index file        #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\vertexPartAssocInfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	CCSVertPartArray.append(struct.unpack('<i', valueBuffer)[0])
	i += 1

i = 0
while i < CCS_VERTEX_COUNT:
	scratchvalue += str(i) + " - "
	scratchvalue += str(CCSVertPartArray[i]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################


################################################
#      Create CCS Normal array file            #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\Normals.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


i = 0
while i < CCS_VERTEX_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSNormals.append( ( struct.unpack('<3f', valueBuffer)[0], struct.unpack('<3f', valueBuffer)[1], struct.unpack('<3f', valueBuffer)[2] ) )
	i += 1


i = 0
while i < CCS_VERTEX_COUNT:
	scratchvalue += str(CCSNormals[i][0]) + " " + str(CCSNormals[i][1]) + " " + str(CCSNormals[i][2]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################


################################################
#      Create Face Info file                   #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\FaceInfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_FACE_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_FACE_COUNT) + "\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################


################################################
#      Skip eye definition section             #
################################################
CCSFilebufferPT += (CCS_FACE_COUNT * 4)



################################################
#            Create Face file                  #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\Faces.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit

i = 0 
while i < CCS_FACE_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+12]
	CCSFilebufferPT += 12
	CCSFaces.append( ( struct.unpack('<3i', valueBuffer)[0], struct.unpack('<3i', valueBuffer)[1], struct.unpack('<3i', valueBuffer)[2] ) )
	i += 1

i = 0 
while i < CCS_FACE_COUNT:
	scratchvalue += str(CCSFaces[i][0]) + " " + str(CCSFaces[i][1]) + " " + str(CCSFaces[i][2]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#        Create bounding box file              #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\BoundingBox.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
CCSFilebufferPT += 4
CCS_BODY_PART_COUNT = struct.unpack('<i', valueBuffer)[0]
scratchvalue += str(CCS_BODY_PART_COUNT) + "\n\n"


i = 0
while i < CCS_BODY_PART_COUNT:
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+24]
	CCSFilebufferPT += 24
	CCSBoundingBox.append( ( struct.unpack('<6f', valueBuffer)[0], struct.unpack('<6f', valueBuffer)[1], struct.unpack('<6f', valueBuffer)[2], struct.unpack('<6f', valueBuffer)[3], struct.unpack('<6f', valueBuffer)[4], struct.unpack('<6f', valueBuffer)[5] ) )
	i += 1

i = 0
while i < CCS_BODY_PART_COUNT:
	scratchvalue += str(CCSBoundingBox[i][0]) + " " + str(CCSBoundingBox[i][1]) + " " + str(CCSBoundingBox[i][2]) + " " + str(CCSBoundingBox[i][3]) + " " + str(CCSBoundingBox[i][4]) + " " + str(CCSBoundingBox[i][5]) + "\n"
	i += 1


owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#        Create face group file                #
################################################
try:
	owfile = open(filedir+filenamenoext+"\\FaceGroups.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


i = 0
while i < CCS_BODY_PART_COUNT:
	CCSFaceGroupArrays.append([])
	valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
	CCSFilebufferPT += 4
	j = struct.unpack('<i', valueBuffer)[0]
	CCSFaceGroupArrays[i].append(j)
	k = 0
	while k < j:
			valueBuffer = CCSFilebuffer[CCSFilebufferPT : CCSFilebufferPT+4]
			CCSFilebufferPT += 4
			CCSFaceGroupArrays[i].append( struct.unpack('<i', valueBuffer)[0] )
			k += 1
	i += 1

i = 0
while i < CCS_BODY_PART_COUNT:
	j = CCSFaceGroupArrays[i][0]
	scratchvalue += str(i) + " : " + str(j) + " : "
	###Details.. to avoid extra comma at the end of the list, put out fist value
	###And add a comma at the beginning of the sub array
	scratchvalue += str( CCSFaceGroupArrays[i][1] )
	k = 2
	while k < j:
		scratchvalue += "," + str( CCSFaceGroupArrays[i][k] )
		k += 1
	scratchvalue += "\n\n"
	i += 1


owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



#Close CCS
orfile.close()



