#
# Black and White OBJ creature model data exporter V0.1
#   	V0.1
#
# This script is mean to export all of the data structures from a creature
# CSK file as text files for analysis. Much of the code is copied from the
# previously written Creature2OBJ.py script. Better commentary on what the
# code does is found in that file.
#   Note that this code is taken from an old version of the Creature2OBJ.py
# script, which is quite a bit different from the current working version.
# This old code still works fine for this analysis script, but
# that other script's commentary will not not exist or not be a good guide
# for what this script's code is doing.. oh well.
#
# A folder named as the creature file provided is created. Each data
# structure is then extracted into text files in this folder. For a
# description of what the data are, see the CSK outline document.
#
# Note that, as I seem to have firmly estanblished that the 'geforce'
# section of the CSK file is ignored, this section is NOT exported by
# this script.
#
# This script is less 'clean' than other scripts.
#
# The "bitwise" array for morphs is skipped
#


import struct
import sys
import os




################################################
#           Variables Used                     #
################################################

#Values considered as 'constants' for the model
#either read in or calculated.
CSK_NUM_MATS = 0
CSK_BODY_PART_MAX = 0
CSK_FACE_ARRAY_ENTRY_COUNT = 0
CSK_VERT_META_ARRAY_SUBARRAY_SIZE = 0
CSK_VERT_META_ARRAY_SIZE = 0
CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE = 0
CSK_BASE_VERTEX_ARRAY_SIZE = 0
CSK_MODEL_VERTEX_COUNT = 0
CSK_MORPH_COUNT = 0
#morph constants change for each morph section
MORPH_VERTEX_ARRAY_SUB_ARRAY_SIZE = 0
MORPH_VERTEX_ARRAY_SIZE = 0



#CSK data structure addresses
#----------------------------
#Face array address
faceArrayLoc = 0
#UV/Meta array address
UVArrayLoc = 0
#Vertex array index
VertexArrayLoc = 0


#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0

#scratch space
scratchvalue = ''

#holds temporary values
valuebuffer = ''

#Hold entire contents of CSK in memory
CSKFilebuffer = ''
#Holds a current address into the CSKFilebuffer used for reading
#variable length portion of the file
CSKFilebufferPT = 0




#Option related Vars
#-------------------
userInput =''
#Negate Vertex Z
NVZ = 1
#Negate UV's (misnomer.. not really negating, just re-arranging)
NUV = 0
#Negate Normal Z
NNZ = 1
#Should the model be welded?
weld = 0
#Should the morphs be extracted?
extractmorphs = 1





#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''



#Holds array describing the 'maaterial' sections and associated
#body-part arrays.
Mats = []


#Holds all the data for each vertex (18 numbers total per entry)
#There should be one index per vertex in the model
VertexTuple = []

#Holds welding substitutions. Each INDEX corresponds to the INDEX of
#a vertex in VertexTuple. The first number in the tuple indicates another
#index. If this value is not equal to the current index, the vertex has
#been welded (was a duplicate and was removed) and indicates which
#vertex it was welded with (that index contains duplicate information)
#The second number in the tuple indicates how many weled (removed)
#vertices there have been up through that index when parsing the array
#beginning at zero. This is needed to keep a correct count of existent
#vertices and to appropriately update the face array's vertex indecies.
#There should be one index per vertex in the model
VertexWeldSub = []

#Holds the UV data and bone association/weight for each vertex
#There should be one index per vertex in the model
UVTuple = []


###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  A CSK file"
	print
	raise SystemExit

filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################





###########################################
#            open CSK                     #
###########################################
try:
	orfile = open(pathtomodelnoext+".csk", 'rb')
except:
	print
	print "ERROR:Cannot open CSK file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit
################################################
################################################



###########################################
#            create folder                #
###########################################
if not os.path.isdir(filenamenoext):
	try:
		os.makedirs(filenamenoext)
	except:
		print
		print "Cannot create output folder"
		print "in the current working directory"
		print "Permission problems?"
		print
		orfile.close()
		raise SystemExit
################################################
################################################








################################################
#           Read CSK header                    #
################################################
try:
	owfile = open(filedir+filenamenoext+"\Header.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit


orfile.seek(0)
CSKFilebuffer = orfile.read()
CSKFilebufferPT = 0
scratchvalue = ''

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+40]
CSKFilebufferPT += 40
scratchvalue += valueBuffer + "\n"

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
scratchvalue += "(Assume int value, acually unknown) " + str(struct.unpack('<i', valueBuffer)[0]) + "\n"

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
scratchvalue += "(Assume int value, acually unknown) " + str(struct.unpack('<i', valueBuffer)[0]) + "\n"

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
scratchvalue += "(Assume int value, acually unknown) " + str(struct.unpack('<i', valueBuffer)[0]) + "\n"

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
scratchvalue += str(struct.unpack('<i', valueBuffer)[0]) + "\n"

CSK_BODY_PART_MAX = struct.unpack_from('<i', valueBuffer)[0]

owfile.write(scratchvalue)
scratchvalue = ''



###If the first section is not 'radeon', Skip it. In that case,
###Gets the size of the first section, then jumps ahead that
###number of bytes.
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+32]
CSKFilebufferPT += 32
if valueBuffer[0] != "r" and valueBuffer[0] != "R":
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	scratchvalue = struct.unpack_from('<i', valueBuffer)[0]
	###Skip the section
	CSKFilebufferPT += scratchvalue
	scratchvalue = ''

	###Read radeon header
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+32]
	CSKFilebufferPT += 32
	scratchvalue += valueBuffer + "\n"
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	scratchvalue += str(struct.unpack_from('<i', valueBuffer)[0]) + "\n"
else:
	###If the section IS 'radeon'
	scratchvalue += valueBuffer + "\n"
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	scratchvalue += str(struct.unpack_from('<i', valueBuffer)[0]) + "\n"


owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()

################################################
################################################





################################################
#           Read CSK material sections         #
################################################
try:
	owfile = open(filedir+filenamenoext+"\Materials.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

arraysRead = 0
#Number of body-part arrays for a particular material section
NUM_PART_ARRAYS = 0

###The Mat array data structure is documented at the top of this file.
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_NUM_MATS = struct.unpack_from('<i', valueBuffer)[0]
scratchvalue += str(CSK_NUM_MATS) + "\n"



i = 0
while i != CSK_NUM_MATS:
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	scratchvalue += str(struct.unpack_from('<i', valueBuffer)[0]) + "\n"

	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+128]
	CSKFilebufferPT += 128
	scratchvalue += "MatNameStart:\n" + valueBuffer + "\n:MatNameEnd\n"

	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+384]
	CSKFilebufferPT += 384
	scratchvalue += "StartUnkownSection:\n" + valueBuffer + "\n:EndUnkownSection\n\n"


	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	NUM_PART_ARRAYS = struct.unpack_from('<i', valueBuffer)[0]
	scratchvalue += str(NUM_PART_ARRAYS) + "\n\n"

	Mats.append(int(NUM_PART_ARRAYS))
	Mats.append([])
	Mats.append([])
	j = 0
	while j != NUM_PART_ARRAYS:
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
		CSKFilebufferPT += 4
		PART_ARRAY_COUNT = struct.unpack_from('<i', valueBuffer)[0]
		scratchvalue += str(PART_ARRAY_COUNT) + "\n"

		Mats[(i*3)+2].append(PART_ARRAY_COUNT)
		### Read the array and the two count numbers
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(PART_ARRAY_COUNT * 4)]
		CSKFilebufferPT += (PART_ARRAY_COUNT * 4)
		Mats[(i*3)+1].append(struct.unpack_from('<'+str(PART_ARRAY_COUNT)+'i', valueBuffer))
		scratchvalue += str( struct.unpack_from('<'+str(PART_ARRAY_COUNT)+'i', valueBuffer) ) + "\n"

		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+8]
		CSKFilebufferPT += 8
		Mats[(i*3)+2].append(struct.unpack_from('<i', valueBuffer)[0])
		Mats[(i*3)+2].append(struct.unpack_from('<2i', valueBuffer)[1])
		scratchvalue += str( struct.unpack_from('<2i', valueBuffer) ) + "\n\n"

		arraysRead += 1
		j += 1
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()

################################################
################################################




################################################
#           Skip CSK Face Array                #
################################################
###Now at the beginning of the Face array.
###For convenience, this should be done later.
###Keep the address
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_FACE_ARRAY_ENTRY_COUNT = struct.unpack_from('<i', valueBuffer)[0]
faceArrayLoc = CSKFilebufferPT
###The entries in this array are words (2 bytes)
CSKFilebufferPT += (CSK_FACE_ARRAY_ENTRY_COUNT * 2)
################################################
################################################




################################################
#           Skip CSK UV/Meta array             #
################################################
###Now at the beginning of the UV arrays. These
###hold UV data and bone associations/weight for
###each vertex. Also skip it for now.
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_VERT_META_ARRAY_SUBARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]
if CSK_VERT_META_ARRAY_SUBARRAY_SIZE != 28:
	print
	print "ERROR: UV array in unknown format"
	print "Each sub-array should be 28-bytes"
	print
	orfile.close()
	owfile.close()
	raise SystemExit
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_VERT_META_ARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]
UVArrayLoc = CSKFilebufferPT
CSKFilebufferPT += CSK_VERT_META_ARRAY_SIZE
################################################
################################################





################################################
#           Read CSK vertex array              #
################################################
###The vertex array contains (X,Y,Z), Normal,
###tangent, and bitangent info per vertex. There
###Are two tan/bitan pairs per vertex (12 numbers)
###but I don't know what effect the second pair has

try:
	owfile = open(filedir+filenamenoext+"\ModelVertexInfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''



#Collect a large string of text to write out for the
#vertex data
vertex = "\n"
#collect a string of the normal data
vertexNormals = "\n"

###Begin extracting the vertices
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+32]
CSKFilebufferPT += 32
scratchvalue += valueBuffer + "\n"

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]
scratchvalue += str(CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE) + "\n"

if CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE != 72:
	print
	print "ERROR:Vertex array in unknown format"
	print "Each sub-array should be 72-bytes"
	print
	orfile.close()
	owfile.close()
	raise SystemExit
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_BASE_VERTEX_ARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]
scratchvalue += str(CSK_BASE_VERTEX_ARRAY_SIZE) + "\n"

CSK_MODEL_VERTEX_COUNT = CSK_BASE_VERTEX_ARRAY_SIZE/CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE
scratchvalue += "(Number of model verts should be: " + str(CSK_MODEL_VERTEX_COUNT) + " )\n\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()

################################################
#           Create Vertex File                 #
################################################
try:
	owfile = open(filedir+filenamenoext+"\BaseVertex.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


###Remember the position of the vertex array
VertexArrayLoc = CSKFilebufferPT

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+CSK_BASE_VERTEX_ARRAY_SIZE]
CSKFilebufferPT += CSK_BASE_VERTEX_ARRAY_SIZE
j = CSK_BASE_VERTEX_ARRAY_SIZE
i = 0
while i < j:
	VertexTuple.append(struct.unpack_from('<18f', valueBuffer[i:i+CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE:1]))
	i += CSK_BASE_VERTEX_ARRAY_SUB_ARRAY_SIZE

i = 0
j = CSK_MODEL_VERTEX_COUNT
while i < j:
	scratchvalue += str(VertexTuple[i][0]) + " " + str(VertexTuple[i][1]) + " " + str(VertexTuple[i][2]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close


###Welding setup. The array VertexWeldSub is initialized to be 1:1.
###Initialize the array
i = 0
while i != CSK_MODEL_VERTEX_COUNT:
	###Create an array of two at each point. The second number will be the 'number of removed
	###vertices up this this index' as determined below.
	VertexWeldSub.append([ int(i), int(0) ] )
	i += 1




################################################
#           Create Normal File                 #
################################################

try:
	owfile = open(filedir+filenamenoext+"\BaseNormals.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

i = 0
j = CSK_MODEL_VERTEX_COUNT
while i < j:
	scratchvalue += str(VertexTuple[i][3]) + " " + str(VertexTuple[i][4]) + " " + str(VertexTuple[i][5]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close


################################################
#           Create Tangent File                #
################################################
try:
	owfile = open(filedir+filenamenoext+"\BaseTanBitan.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

i = 0
j = CSK_MODEL_VERTEX_COUNT
while i < j:
	scratchvalue += str(VertexTuple[i][6]) + " " + str(VertexTuple[i][7]) + " " + str(VertexTuple[i][8]) + str(VertexTuple[i][9]) + " " + str(VertexTuple[i][10]) + " " + str(VertexTuple[i][11]) + str(VertexTuple[i][12]) + " " + str(VertexTuple[i][13]) + " " + str(VertexTuple[i][14]) + str(VertexTuple[i][15]) + " " + str(VertexTuple[i][16]) + " " + str(VertexTuple[i][17]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close



#delete variables no longer needed
del vertex
del vertexNormals
################################################
################################################






################################################
#           Read CSK UV/Meta array             #
################################################
###The UV array contains UV data and bone
###association/weights per vertex.
###CURRENTLY, the bone data is ignored

try:
	owfile = open(filedir+filenamenoext+"\BaseUVinfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

scratchvalue += str(CSK_VERT_META_ARRAY_SUBARRAY_SIZE) + "\n"
scratchvalue += str(CSK_VERT_META_ARRAY_SIZE) + "\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close


################################################
#           Create UV file                     #
################################################

try:
	owfile = open(filedir+filenamenoext+"\BaseUV.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


#Collect a large string of text to write out for the
#UV data
vertexUV = "\n"



###Go back and get the UV texture coordinate array
CSKFilebufferPT = UVArrayLoc
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+CSK_VERT_META_ARRAY_SIZE]
CSKFilebufferPT += CSK_VERT_META_ARRAY_SIZE

j = CSK_VERT_META_ARRAY_SIZE
i = 0
while i != j:
	UVTuple.append(struct.unpack_from('<2f4h3f', valueBuffer[i:i+CSK_VERT_META_ARRAY_SUBARRAY_SIZE:1]))
	i += CSK_VERT_META_ARRAY_SUBARRAY_SIZE
i = 0


j = CSK_MODEL_VERTEX_COUNT
i = 0
while i < j:
	scratchvalue += str(UVTuple[i][0]) + " " + str(UVTuple[i][1]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close



################################################
#           Create Bone info file              #
################################################
try:
	owfile = open(filedir+filenamenoext+"\BaseBoneinfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


j = CSK_MODEL_VERTEX_COUNT
i = 0
while i < j:
	scratchvalue += str(UVTuple[i][2]) + " " + str(UVTuple[i][3]) + " " + str(UVTuple[i][4]) + " " + str(UVTuple[i][5]) + " " + str(UVTuple[i][6]) + " " + str(UVTuple[i][7]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close

################################################
################################################







################################################
#   Reconstruct faces from face index array    #
################################################
#Holds the entire face array as an unpacked, parsable data structure
faceTuple = ''
#Holds an array of text strings.. Each main index represents a single
#body part and the sub-indexes (each text string) will contain the OBJ
#group declaration for that part, its name, and then a collection of
#all the faces (and their neccesary info) in that group.
partFaceArray = []
#Has ints up to the number of body parts. Will hold a running count of
#the weight to each body part (bone) for each face while finding which
#group each face should belong to.
weightHelp = []
numArrays = 0
endArrayEntryNum = 0
highestWeightValue = -1
highestEntryNum = -1
fIndex = 0
fCount = 0
#index plus 1, plus 0 (current index), and minus 1
indexP1 = 0
indexP0 = 0
indexM1 = 0
###Get the faces. this is a little complicted. I decided (for now)
###to only assign one body part to each face. If I am able to extract
###bones, the way this works may change since each vertex will be
###weighted to multiple body parts. This will require a non-stanrd
###OBJ, though, or a different format. See note at the top.
###
### NOTE!!! Anywhere 'facetuple' is referenced, VertexWeldSub must be
### used to substitute the value of facetuple, since facetuple refers
### To vertex indeces
###
###Create "CSK_BODY_PART_MAX" part arrays. Faces will be added to these
###Note that these will be indexed from 0, so the actual index numbers
###will be CSK_BODY_PART_MAX - 1.
###Also create weighthelp array. This is used to collect vertex
###weights to help detemine face group




try:
	owfile = open(filedir+filenamenoext+"\FacesInfo.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

scratchvalue += str(CSK_FACE_ARRAY_ENTRY_COUNT) + "\n"

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close


############################################################


try:
	owfile = open(filedir+filenamenoext+"\Faces.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''

try:
	owfile2 = open(filedir+filenamenoext+"\RAW-Faces.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue2 = ''

try:
	owfile3 = open(filedir+filenamenoext+"\RAW-Faces-Triplets.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue3 = ''



CSKFilebufferPT = faceArrayLoc

###Each entry is a word, 2-bytes
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(CSK_FACE_ARRAY_ENTRY_COUNT * 2)]
CSKFilebufferPT += CSK_FACE_ARRAY_ENTRY_COUNT * 2

#create an array of *words* of the entire face array
faceTuple = struct.unpack_from('<' + str(CSK_FACE_ARRAY_ENTRY_COUNT) + 'h', valueBuffer)

################################################
#             create RAW face file             #
################################################

j = CSK_FACE_ARRAY_ENTRY_COUNT
i = 0
while i < j:
	scratchvalue2 += str(faceTuple[i]) + "\n"
	i += 1

owfile2.write(scratchvalue2)
scratchvalue2 = ''
owfile2.close


############################################################


i = 0
while i != CSK_BODY_PART_MAX:
	partFaceArray.append("g BodyPart_" + str(i) + "\n")
	weightHelp.append(int(0))
	i += 1
i = 0
while i != CSK_NUM_MATS:
	###numArrays will be the number of part arrays in this Material
	numArrays = Mats[i*3]
	j = 0
	while j != numArrays:
		###k will be the beginning index of this part-array's listings in the face array
		k = Mats[(i*3)+2][(j*3)+1]
		###endArrayEntryNum will be the last index number in the face arary for this
		###part-array
		endArrayEntryNum = Mats[(i*3)+2][(j*3)+2] + k
		###Faces will be triplets, (index-1, index, index+1). For index-1 and index+1 to
		###exist correctly, the reading must start at k+1 and end at endArrayEntryNum-1
		k = k + 1
		endArrayEntryNum = endArrayEntryNum - 1
		h = 0
		while k != endArrayEntryNum:
			m = 0
			###reset the weight array buckets
			while m != CSK_BODY_PART_MAX:
				weightHelp[m] = 0
				m += 1
			###reset highest index weight tracker
			highestWeightValue = -1
			highestEntryNum = -1

################################################
#  RAW face file w/ triplet collection point   #
################################################
			scratchvalue3 += str(faceTuple[k-1]) + " " + str(faceTuple[k]) + " " + str(faceTuple[k+1])


			###As long as any of the numbers in the triplet are not the same...
			if VertexWeldSub[faceTuple[k]][0] != VertexWeldSub[faceTuple[k-1]][0] and VertexWeldSub[faceTuple[k]][0] != VertexWeldSub[faceTuple[k+1]][0] and VertexWeldSub[faceTuple[k-1]][0] != VertexWeldSub[faceTuple[k+1]][0]:
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k]][2])/3]] += UVTuple[VertexWeldSub[faceTuple[k]][0]][6]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k]][3])/3]] += UVTuple[VertexWeldSub[faceTuple[k]][0]][7]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k]][4])/3]] += UVTuple[VertexWeldSub[faceTuple[k]][0]][8]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k-1]][2])/3]] += UVTuple[VertexWeldSub[faceTuple[k-1]][0]][6]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k-1]][3])/3]] += UVTuple[VertexWeldSub[faceTuple[k-1]][0]][7]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k-1]][4])/3]] += UVTuple[VertexWeldSub[faceTuple[k-1]][0]][8]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k+1]][2])/3]] += UVTuple[VertexWeldSub[faceTuple[k+1]][0]][6]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k+1]][3])/3]] += UVTuple[VertexWeldSub[faceTuple[k+1]][0]][7]
				weightHelp[Mats[(i*3)+1][j][(UVTuple[faceTuple[k+1]][4])/3]] += UVTuple[VertexWeldSub[faceTuple[k+1]][0]][8]
				m = 0
				while m != CSK_BODY_PART_MAX:
					if weightHelp[m] > highestWeightValue:
						highestWeightValue = weightHelp[m]
						highestEntryNum = m
					m += 1
				###index plus 1, index plus 0 (current index), index minus 1. The dereferences are so long.. do it once
				###So.. The vertex, and if it's substituted (welded), the welded index, minus the number of removed
				###vertecies up to that point where the substituted vertex is, since the whole vertex list will be
				###shifted down by that amount. IE if you remove 5 vertices, the last vertex will have index X-5 instead
				###of its original index X
				###Also, the normal will need to be the ORIGNAL normal or else the UVs will scatter all over the place
				###at the seams. Each welded vertex will have mor than one normal
				### (+1) because OBJ indexes for vertex/normal/ etc.. start at 1, not 0.
				indexP1 = VertexWeldSub[faceTuple[k+1]][0]-VertexWeldSub[VertexWeldSub[faceTuple[k+1]][0]][1]+1
				indexP0 = VertexWeldSub[faceTuple[k]][0]-VertexWeldSub[VertexWeldSub[faceTuple[k]][0]][1]+1
				indexM1 = VertexWeldSub[faceTuple[k-1]][0]-VertexWeldSub[VertexWeldSub[faceTuple[k-1]][0]][1]+1
				indexP1N = faceTuple[k+1]+1
				indexP0N = faceTuple[k]+1
				indexM1N = faceTuple[k-1]+1
				if h == 0:
					partFaceArray[highestEntryNum] += "f " + str(indexM1)+"/"+str(indexM1N)+"/"+str(indexM1)+" "+str(indexP0)+"/"+str(indexP0N)+"/"+str(indexP0)+" "+str(indexP1)+"/"+str(indexP1N)+"/"+str(indexP1) + "\n"
#(-1) on these below sine 1 was added above, intending to be written to as an OBJ index
################################################
#  RAW face file w/ triplet collection point   #
################################################
					scratchvalue3 += " * " + str(indexM1-1) + " " + str(indexP0-1) + " " + str(indexP1-1)
################################################
#         face file collection point           #
################################################
					scratchvalue += str(indexM1-1) + " " + str(indexP0-1) + " " + str(indexP1-1) + "\n"
				else:
#					partFaceArray[highestEntryNum] += "f " + str(indexP1)+"/"+str(indexP1)+"/"+str(indexP1)+" "+str(indexP0)+"/"+str(indexP0)+"/"+str(indexP0)+" "+str(indexM1)+"/"+str(indexM1)+"/"+str(indexM1) + "\n"
					partFaceArray[highestEntryNum] += "f " + str(indexP0)+"/"+str(indexP0N)+"/"+str(indexP0)+" "+str(indexM1)+"/"+str(indexM1N)+"/"+str(indexM1)+" "+str(indexP1)+"/"+str(indexP1N)+"/"+str(indexP1) + "\n"
################################################
#  RAW face file w/ triplet collection point   #
################################################
					scratchvalue3 += " * " + str(indexP0-1) + " " + str(indexM1-1) + " " + str(indexP1-1)
################################################
#         face file collection point           #
################################################
					scratchvalue += str(indexP0-1) + " " + str(indexM1-1) + " " + str(indexP1-1) + "\n"

			h = (h + 1) % 2
			k += 1
################################################
#  RAW face file w/ triplet collection point   #
################################################
			scratchvalue3 += "\n"
		j += 1
	i += 1


owfile3.write(scratchvalue3)
scratchvalue3 = ''
owfile3.close

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close


################################################
################################################





################################################
#                 Clean up                     #
################################################
###Done writing out base file. Close it.
owfile.close()

###Delete stuff no longer needed
del faceTuple
del weightHelp
del UVTuple
del Mats
del numArrays
del endArrayEntryNum
del highestWeightValue
del highestEntryNum
del fIndex
del fCount
del indexP1
del indexP0
del indexM1
################################################
################################################





################################################
#               Extract Morphs                 #
################################################
#This basically works the same way as the vertex
#read including the vertextuple array and the
#large string variables that collect the info to
#write out. The main addition is the bitwise
#description of which vertices displace per morph.
#A shifting window of 8-bits is AND-ed with the
#current bitwise-describing byte to test for
#each vertex's displacement.


#bitArray declared below, as a type 'buffer'
###All these are initialized below for each morph
#morphVertexArrayCount = 0
#morphBitArrayCount = 0
#vertex = "\n"
#vertexNormals = "\n"
#morphname = ''
#loopshiftcount = 8
###Same structre as the VertexTuple. all data are DISPLACEMENT values
###rather than actual values. In other words x = 0.0023 means X displaced
###by that much from the base position. Same for normals,tan,bitan
#VertexMorphTuple = []
###Begin Extracting the morphs. Much of the code from above is simply copy-pasted..
if extractmorphs == 1:

	###Go to the end of the base-morph vertex array
	CSKFilebufferPT = VertexArrayLoc+CSK_BASE_VERTEX_ARRAY_SIZE
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	CSK_MORPH_COUNT = struct.unpack_from('<i', valueBuffer)[0]

################################################
#            Write morph-count file            #
################################################
	try:
		owfile = open(filedir+filenamenoext+"\\" + str(CSK_MORPH_COUNT) + "-Morphs", 'w')
	except:
		print
		print "ERROR: Cannot open text file for writing"
		print "in the folder"
		print str(filedir)+filenamenoext
		print "Persmission problems?"
		print
		orfile.close()
		raise SystemExit
	scratchvalue = ''

	scratchvalue += str(CSK_MORPH_COUNT)

	owfile.write(scratchvalue)
	scratchvalue = ''
	owfile.close()




	i = 0
	while i != CSK_MORPH_COUNT:
		###This is needed since loop indices will not keep track of the 'read' (past-tense) position
		###of the morph vertex array.
		morphVertexArrayCount = 0
		morphBitArrayCount = 0
		VertexMorphTuple = []
		morphname = ''
		vertex = "\n"
		vertexNormals = "\n"
		###Get the name
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+32]
		CSKFilebufferPT += 32
		###Python string functions seem to change so often, I think it is better to avoid using them...
		j = 0
		while j != 32:
			if valueBuffer[j] != '\x00':
				morphname += valueBuffer[j]
			j += 1
################################################
#           write morph info file              #
################################################
		try:
			owfile = open(filedir+filenamenoext+"\\" + str(morphname) + "-Info.txt", 'w')
		except:
			print
			print "ERROR: Cannot open text file for writing"
			print "in the folder"
			print str(filedir)+filenamenoext
			print "Persmission problems?"
			print
			orfile.close()
			raise SystemExit
		scratchvalue = ''

		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+8]
		CSKFilebufferPT += 8
		MORPH_VERTEX_ARRAY_SUB_ARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]


		scratchvalue += str(MORPH_VERTEX_ARRAY_SUB_ARRAY_SIZE) + "\n"
		scratchvalue += str( struct.unpack_from('<2i', valueBuffer)[1] ) + "\n"

		owfile.write(scratchvalue)
		scratchvalue = ''
		owfile.close()



		###There will be 1 byte per vertex. this is actulally a mistake.. the array is a bit-wise reference.
		bitArray = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+CSK_MODEL_VERTEX_COUNT]
		CSKFilebufferPT += CSK_MODEL_VERTEX_COUNT

		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
		CSKFilebufferPT += 4
		###Since the morph vertex array will not include all of the vertices, this number is important..
		MORPH_VERTEX_ARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]

		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+MORPH_VERTEX_ARRAY_SIZE]
		CSKFilebufferPT += MORPH_VERTEX_ARRAY_SIZE

		j = MORPH_VERTEX_ARRAY_SIZE
		k = 0
		###Read in and process the morph vertex array.. create a tuple in the same way as done for the base arrays
		while k != j:
			VertexMorphTuple.append(struct.unpack_from('<18f', valueBuffer[k:k+MORPH_VERTEX_ARRAY_SUB_ARRAY_SIZE:1]))
			k += MORPH_VERTEX_ARRAY_SUB_ARRAY_SIZE
		j = 0


################################################
#     write displacement morph value file      #
################################################
		try:
			owfile = open(filedir+filenamenoext+"\\" + str(morphname) + "-Displacements.txt", 'w')
		except:
			print
			print "ERROR: Cannot open text file for writing"
			print "in the folder"
			print str(filedir)+filenamenoext
			print "Persmission problems?"
			print
			orfile.close()
			raise SystemExit
		scratchvalue = ''

		
		j = len(VertexMorphTuple)
		k = 0
		while k < j:
			scratchvalue += str(VertexMorphTuple[k][0]) + " " + str(VertexMorphTuple[k][1]) + " " + str(VertexMorphTuple[k][2]) + " " + str(VertexMorphTuple[k][3]) + " " + str(VertexMorphTuple[k][4]) + " " + str(VertexMorphTuple[k][5]) + " " + str(VertexMorphTuple[k][6]) + " " + str(VertexMorphTuple[k][7]) + " " + str(VertexMorphTuple[k][8]) + " " + str(VertexMorphTuple[k][9]) + " " + str(VertexMorphTuple[k][10]) + " " + str(VertexMorphTuple[k][11]) + " " + str(VertexMorphTuple[k][12]) + " " + str(VertexMorphTuple[k][13]) + " " + str(VertexMorphTuple[k][14]) + " " + str(VertexMorphTuple[k][15]) + " " + str(VertexMorphTuple[k][16]) + " " + str(VertexMorphTuple[k][17]) + "\n"
			k += 1

		owfile.write(scratchvalue)
		scratchvalue = ''
		owfile.close()
############################################################


		###This loop will parse the bitwise references by bit-wise AND'ing a shifting mask with the byte
		###read, in a inner loop of 8-count (8 bits per byte). Note that the bit on the right is the first
		###to be tested. So, vertex specifications are [ 7 6 5 4 3 2 1 0 ], [ 15 14 13 12 11 10 9 8 ], ...
		###"Less Than" is used since the loop incrementer will be adding '8' instead of '1'
		while j < CSK_MODEL_VERTEX_COUNT:
			###This value will be shifted once per loop
			loopshiftcount = 8
			###I screwed up some indexing plans, so this value is a crutch.. oh well.
			currentVnum = j
			bintest = 0b00000001
			binval = struct.unpack('<B', bitArray[morphBitArrayCount])[0]
			###The very last byte will often have less than 8 relavant bits. If the total model vert count minus the
			###current outer loop index is less than 8, there are only that number of bits left to test
			if (CSK_MODEL_VERTEX_COUNT-j) < loopshiftcount:
				loopshiftcount = CSK_MODEL_VERTEX_COUNT-j
			while loopshiftcount != 0:
				###get the current BASE VERTEX ARRAY index.. oi, this gets complicated. Time for some Weird Al. Perform This Way!
				###bitwise AND with the shifting mask. If true, this vertex has morphed.
				###Remember that welding must still be done.
				###IF IT MORPHED: Calculate the displacement and write out the new vertex position
				###IF NOT MORHPED: Write out the original vertex position
				if (bintest & binval):
					###Only add this vertex if it wasn't 'removed' in welding. Still increment morphVertexArrayCount even if it was. this is
					###checked inside this 'if' since this count will still need to be incremented.
					###ALSO! remember that the vertex Z and the normal Z may need to be negated
					if VertexWeldSub[currentVnum][0] == currentVnum:
						vertex += "v " + str(format(VertexTuple[currentVnum][0]+VertexMorphTuple[morphVertexArrayCount][0], ' .8f')) + " " + str(format(VertexTuple[currentVnum][1]+VertexMorphTuple[morphVertexArrayCount][1], ' .8f')) + " " + str(format((VertexTuple[currentVnum][2]+VertexMorphTuple[morphVertexArrayCount][2])*NVZ, ' .8f')) + "\n"
						vertexNormals += "vn " + str(format(VertexTuple[currentVnum][3]+VertexMorphTuple[morphVertexArrayCount][3], ' .8f')) + " " + str(format(VertexTuple[currentVnum][4]+VertexMorphTuple[morphVertexArrayCount][4], ' .8f')) + " " + str(format((VertexTuple[currentVnum][5]+VertexMorphTuple[morphVertexArrayCount][5])*NNZ, ' .8f')) + "\n"
					morphVertexArrayCount += 1
				else:
					if VertexWeldSub[currentVnum][0] == currentVnum:
						vertex += "v " + str(format(VertexTuple[currentVnum][0], ' .8f')) + " " + str(format(VertexTuple[currentVnum][1], ' .8f')) + " " + str(format((VertexTuple[currentVnum][2]*NVZ), ' .8f')) + "\n"
						vertexNormals += "vn " + str(format(VertexTuple[currentVnum][3], ' .8f')) + " " + str(format(VertexTuple[currentVnum][4], ' .8f')) + " " + str(format((VertexTuple[currentVnum][5]*NNZ), ' .8f')) + "\n"
				bintest = bintest << 1
				currentVnum += 1
				loopshiftcount -= 1
			###Prepare to read the next byte in the bit array
			morphBitArrayCount += 1
			###We just read 8 vertices in the inner loop, not 1
			j += 8
		i += 1
################################################
################################################







################################################
#     Create Edge connectivity array           #
################################################
edgecount = 0
edgeTuple = []

try:
	owfile = open(filedir+filenamenoext+"\EdgeConnectivity.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


###Skip name. Ignore this as info, should always be the same
CSKFilebufferPT += 32

###Skip size
CSKFilebufferPT += 4

###Count of entries
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
edgecount = struct.unpack('<i', valueBuffer)[0]

###4 ints per entry, 4 bytes each
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(edgecount*4*4)]
CSKFilebufferPT += edgecount*4*4


j = 0
i = 0
while i < edgecount:
	edgeTuple.append(struct.unpack_from('<4i', valueBuffer[j:j+16:1]))
	i += 1
	j += 16

i = 0
while i < edgecount:
	scratchvalue += str(edgeTuple[i][0]) + " " + str(edgeTuple[i][1]) + " " + str(edgeTuple[i][2]) + " " + str(edgeTuple[i][3]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#     Create Edge 96 array                     #
################################################
facecount = 0
edgeTuple = []

try:
	owfile = open(filedir+filenamenoext+"\Connectivity96.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


###Skip name. Ignore this as info, should always be the same
CSKFilebufferPT += 32

###Skip size
CSKFilebufferPT += 4

###Count of entries
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
facecount = struct.unpack('<i', valueBuffer)[0]

###8 ints per entry, 4 bytes each
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(facecount*8*4)]
CSKFilebufferPT += facecount*8*4


j = 0
i = 0
while i < facecount:
	edgeTuple.append(struct.unpack_from('<8i', valueBuffer[j:j+32:1]))
	i += 1
	j += 32

i = 0
while i < facecount:
	scratchvalue += str(edgeTuple[i][0]) + " " + str(edgeTuple[i][1]) + " " + str(edgeTuple[i][2]) + " " + str(edgeTuple[i][3]) + " " + str(edgeTuple[i][4]) + " " + str(edgeTuple[i][5]) + " " + str(edgeTuple[i][6]) + " " + str(edgeTuple[i][7]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



################################################
#     Create Edge 256 array                    #
################################################
facecount = 0
edgeTuple = []

try:
	owfile = open(filedir+filenamenoext+"\Connectivity256.txt", 'w')
except:
	print
	print "ERROR: Cannot open text file for writing"
	print "in the folder"
	print str(filedir)+filenamenoext
	print "Persmission problems?"
	print
	orfile.close()
	raise SystemExit
scratchvalue = ''


###Skip name. Ignore this as info, should always be the same
CSKFilebufferPT += 32

###Skip size
CSKFilebufferPT += 4

###Count of entries
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
facecount = struct.unpack('<i', valueBuffer)[0]

###8 ints per entry, 4 bytes each
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(facecount*8*4)]
CSKFilebufferPT += facecount*8*4


j = 0
i = 0
while i < facecount:
	edgeTuple.append(struct.unpack_from('<8i', valueBuffer[j:j+32:1]))
	i += 1
	j += 32

i = 0
while i < facecount:
	scratchvalue += str(edgeTuple[i][0]) + " " + str(edgeTuple[i][1]) + " " + str(edgeTuple[i][2]) + " " + str(edgeTuple[i][3]) + " " + str(edgeTuple[i][4]) + " " + str(edgeTuple[i][5]) + " " + str(edgeTuple[i][6]) + " " + str(edgeTuple[i][7]) + "\n"
	i += 1

owfile.write(scratchvalue)
scratchvalue = ''
owfile.close()
################################################
################################################



#Close CSK
orfile.close()



