#
# Black and White 2 creature model
# Normal, Tangent, and Bitangent test V0.1
#
#	V0.1
#
#
# This script reads in a CSK file, gathers the vertex, normal, 
# tanget, and bitangent information and outputs an OBJ with
# the selected components as 'lines'. The idea is to allow
# testing the placement of the normals, tangents, and
# bitangents in a CSK.
#
#	TO DO
#	 - Would it be useful to output the morph values?
#	   Morphs hold displacement values for the base
#	   normal/tan/bitan




import struct
import math
import sys
import os

################################################
#           Variables Used                     #
################################################

#Values considered as 'constants' for the model
#either read in or calculated.
MORPH_COUNT = 0
CSK_BODY_PART_MAX = 0
CSK_NUM_MATS = 0
CSK_FACE_ARRAY_ENTRY_COUNT = 0
CSK_VERT_META_ARRAY_SUBARRAY_SIZE = 0
CSK_VERT_META_ARRAY_SIZE = 0
CSK_BASE_VERTEX_ARRAY_SIZE = 0
CSK_MODEL_VERTEX_COUNT = 0




#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0




#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''




#Option related Vars
#-------------------
userInput =''
#Export normals
N = 0
#Export 1st set of Tangents
T1 = 0
#Export 2nd set of Tangents
T2 = 0
#Export 1st set of Bitangents
BT1 = 0
#Export 2nd set of Bitangents
BT2 = 0


#scratch space
scratchvalue = ''
#holds temporary values
valuebuffer = ''


#Hold entire contents of original CSK in memory
CSKFilebuffer = ''
#Holds a current address into the CSKFilebuffer used for reading
#variable length portion of the file
CSKFilebufferPT = 0




#holds verbatim the 384-byte section, who's purpose I've never really been able to discover
CSK384byteArray = ''





#Holds data describing the original CSK's material sections, *plus*
#the min and max vertex index numbers of the associated section of
#the face array (not present in Creature2OBJ)
CSKMats = []

#Holds array of all UV points read in from the original CSK
CSKUV = []

#Holds the vertices read from the original CSK model
CSKVerts = []

#Holds normal data for each vertex read from the original CSK file. The
#indexes match the index from the vertex array
CSKNormals = []

#Holds tangent and bitangent data for each vertex read from the original
#CSK file. Indexes match the index from the vertex array
CSKTANBI = [] 




#New OBJ vertex list
newVerts = []
#New OBJ Faces (lines)
newFaces = []



#Holds temporarily unpacked arrays while data is processed
tempArray = []


################################################
################################################





###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  A CSK file"
	print
	raise SystemExit


filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################




###########################################
#  Open CSK file and create output OBJ    #
###########################################
try:
	orfile = open(pathtomodelnoext+".CSK", 'rb')
except:
	print
	print "ERROR:Cannot open CSK file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit


try:
	owfile = open(pathtomodelnoext+"-CSKNormalLines.obj", 'w')
except:
	print
	print "ERROR:Cannot open file for writing."
	print "Disk Full? Directory read-only?"
	print
	orfile.close()
	raise SystemExit
################################################
################################################




################################################
#                 Options                      #
################################################
print
print
print "Select which items to export"
print "----------------------------"
print
print "Normals ?"
print "  1) Y"
print "  2) N"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "1":
	N = 1
userInput = ''

print "1st Tangent ?"
print "  1) Y"
print "  2) N"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "1":
	T1 = 1
userInput = ''

print "1st Bitangent ?"
print "  1) Y"
print "  2) N"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "1":
	BT1 = 1
userInput = ''

print "2nd Tangent ?"
print "  1) Y"
print "  2) N"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "1":
	T2 = 1
userInput = ''

print "2nd Bitangent ?"
print "  1) Y"
print "  2) N"
print
while userInput != "1" and userInput != "2":
	userInput = raw_input('--> ')
	if userInput != "1" and userInput != "2":
		print "Input is not correct"
if userInput == "1":
	BT2 = 1
userInput = ''
################################################
################################################




################################################
#    Read existing CSK file data structures    #
################################################
#Read in the CSK file's data
#This is very much copy-pasted from the twin
#Creature2OBJ script
matName = ''



################################################
#           Read CSK file and get header       #
################################################
CSKFilebuffer = orfile.read()
CSKFilebufferPT = 52

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4

CSK_BODY_PART_MAX = struct.unpack_from('<i', valueBuffer)[0]


###Get the radeon section only ('geforce' seems to be skipped)
###Go to the start of the first section, see if
###is the 'radeon' section. If not, skip it
###In that case,
###Gets the size of the first section, then jumps ahead that
###number of bytes.
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+32]
CSKFilebufferPT += 32

if valueBuffer[0] != "r" and valueBuffer[0] != "R":
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	###Skip the section
	CSKFilebufferPT += struct.unpack_from('<i', valueBuffer)[0]
	###Skip the second section's header
	CSKFilebufferPT += 36
else:
	###If the section IS 'radeon', skip the section-size value
	CSKFilebufferPT += 4

################################################
#           Read CSK material sections         #
################################################
NUM_PART_ARRAYS = 0

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_NUM_MATS = struct.unpack_from('<i', valueBuffer)[0]

i = 0
while i != CSK_NUM_MATS:
	CSKFilebufferPT += 4
	matName = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+128]
	CSKFilebufferPT += 128
	if i == 0:
		CSK384byteArray = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+384]
		CSKFilebufferPT += 384
	else:
		CSKFilebufferPT += 384
	valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
	CSKFilebufferPT += 4
	NUM_PART_ARRAYS = struct.unpack_from('<i', valueBuffer)[0]
	CSKMats.append( ( int(NUM_PART_ARRAYS), matName ) )
	CSKMats.append([])
	CSKMats.append([])
	CSKMats.append([])
	j = 0
	while j != NUM_PART_ARRAYS:
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
		CSKFilebufferPT += 4
		PART_ARRAY_COUNT = struct.unpack_from('<i', valueBuffer)[0]
		### Read the array and the two count numbers
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+(PART_ARRAY_COUNT * 4)]
		CSKFilebufferPT += PART_ARRAY_COUNT * 4
		CSKMats[(i*4)+1].append(struct.unpack_from('<'+str(PART_ARRAY_COUNT)+'i', valueBuffer))
		valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+8]
		CSKFilebufferPT += 8
		CSKMats[(i*4)+2].append([])
		CSKMats[(i*4)+2][j].append(PART_ARRAY_COUNT)
		CSKMats[(i*4)+2][j].append(struct.unpack_from('<i', valueBuffer)[0])
		CSKMats[(i*4)+2][j].append(struct.unpack_from('<2i', valueBuffer)[1])
		j += 1
	i += 1

################################################
#           Skip the CSK face array            #
################################################
# There is no reason to reas this in in this script


valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4
CSK_FACE_ARRAY_ENTRY_COUNT = struct.unpack_from('<i', valueBuffer)[0]

###Skip the array
CSKFilebufferPT += CSK_FACE_ARRAY_ENTRY_COUNT * 2



################################################
#           Read CSK UV/Meta array             #
################################################
#In this script, there is no need to read in the bone
#information

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+8]
CSKFilebufferPT += 8
CSK_VERT_META_ARRAY_SUBARRAY_SIZE = struct.unpack_from('<i', valueBuffer)[0]
CSK_VERT_META_ARRAY_SIZE = struct.unpack_from('<2i', valueBuffer)[1]
CSK_MODEL_VERTEX_COUNT = CSK_VERT_META_ARRAY_SIZE / CSK_VERT_META_ARRAY_SUBARRAY_SIZE

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+CSK_VERT_META_ARRAY_SIZE]
CSKFilebufferPT += CSK_VERT_META_ARRAY_SIZE

i = 0
while i < CSK_VERT_META_ARRAY_SIZE:
	tempArray.append(struct.unpack_from('<2f4h3f', valueBuffer[i:i+CSK_VERT_META_ARRAY_SUBARRAY_SIZE:1]))
	i += CSK_VERT_META_ARRAY_SUBARRAY_SIZE

i = 0
while i != CSK_MODEL_VERTEX_COUNT:
	CSKUV.append( ( tempArray[i][0] , tempArray[i][1] ) )
	i += 1



################################################
#           Read CSK vertex array              #
################################################
CSKFilebufferPT += 36

valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+4]
CSKFilebufferPT += 4

CSK_BASE_VERTEX_ARRAY_SIZE = struct.unpack('<i', valueBuffer)[0]
valueBuffer = CSKFilebuffer[CSKFilebufferPT : CSKFilebufferPT+CSK_BASE_VERTEX_ARRAY_SIZE]
CSKFilebufferPT += CSK_BASE_VERTEX_ARRAY_SIZE

j = CSK_BASE_VERTEX_ARRAY_SIZE
i = 0
while i < j:
	CSKVerts.append(struct.unpack_from('<3f', valueBuffer[i:i+12:1]))
	CSKNormals.append(struct.unpack_from('<3f', valueBuffer[i+12:i+24:1]))
	CSKTANBI.append(struct.unpack_from('<12f', valueBuffer[i+24:i+72:1]))
	i += 72



################################################
#          Clean up from CSK read              #
################################################
valueBuffer = ''
tempArray = []

del CSKFilebuffer
del CSKFilebufferPT

del matName

orfile.close()
################################################
################################################
################################################
################################################








################################################
#               Create OBJ File                #
################################################
NewX = 0
NewY = 0
NewZ = 0
vertexCount = 0
currentMainVert = 0
filelength = 0

i = 0
while i < CSK_MODEL_VERTEX_COUNT:
	newVerts.append( ( CSKVerts[i][0], CSKVerts[i][1], CSKVerts[i][2] ) )
	currentMainVert = vertexCount
	vertexCount += 1
	if N:
		NewX = CSKNormals[i][0]
		NewY = CSKNormals[i][1]
		NewZ = CSKNormals[i][2]
		newVerts.append( ( CSKVerts[i][0]+NewX, CSKVerts[i][1]+NewY, CSKVerts[i][2]+NewZ ) )
		###Add new faces (lines)
		newFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
	if T1:
		NewX = CSKTANBI[i][0]
		NewY = CSKTANBI[i][1]
		NewZ = CSKTANBI[i][2]
		newVerts.append( ( CSKVerts[i][0]+NewX, CSKVerts[i][1]+NewY, CSKVerts[i][2]+NewZ ) )
		###Add new faces (lines)
		newFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
	if T2:
		NewX = CSKTANBI[i][6]
		NewY = CSKTANBI[i][7]
		NewZ = CSKTANBI[i][8]
		newVerts.append( ( CSKVerts[i][0]+NewX, CSKVerts[i][1]+NewY, CSKVerts[i][2]+NewZ ) )
		###Add new faces (lines)
		newFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
	if BT1:
		NewX = CSKTANBI[i][3]
		NewY = CSKTANBI[i][4]
		NewZ = CSKTANBI[i][5]
		newVerts.append( ( CSKVerts[i][0]+NewX, CSKVerts[i][1]+NewY, CSKVerts[i][2]+NewZ ) )
		###Add new faces (lines)
		newFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
	if BT2:
		NewX = CSKTANBI[i][9]
		NewY = CSKTANBI[i][10]
		NewZ = CSKTANBI[i][11]
		newVerts.append( ( CSKVerts[i][0]+NewX, CSKVerts[i][1]+NewY, CSKVerts[i][2]+NewZ ) )
		###Add new faces (lines)
		newFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
	i += 1


###########################################
#             Write new OBJ               #
###########################################
filelength = len(newVerts)
i = 0
while i < filelength:
	scratchvalue += "v " + str(newVerts[i][0]) + " " + str(newVerts[i][1]) + " " + str(newVerts[i][2]) + "\n"
	i += 1


scratchvalue += "\n\n"

filelength = len(newFaces)
i = 0
while i < filelength:
	###(+1) since OBJ indexes start from 1, not 0
	scratchvalue += "f " + str(newFaces[i][0]+1) + "// " + str(newFaces[i][1]+1) + "// \n"
	i += 1

owfile.write(scratchvalue)

owfile.close()











