#
# OBJ Normal exporter V0.1
#   	V0.1
#     TO DO:
#        - Add a 'weight' option to set the length of the lines?
#
# This script reads in an OBJ file and generates 'lines' that correspond
# to the normals specified for each vertex. The idea is to be able to see
# what the normals in an OBJ actually are. Often, 3D programs
# will generate their own normals or will not otherwise display the actual
# normals specified in an imported file. By created lines corresponding to
# to the normals, the resulting OBJ can be imported into a program like
# blender to see the actual normals specified in the OBJ.
#
# Unlike the other OBJ reading in some other scripts, there is no averaging
# or normalizing. If a vertex has more than one normal, it will show up.
#

import struct
import sys
import os




################################################
#           Variables Used                     #
################################################




#Various indecies
#----------------
i = 0
j = 0
k = 0
m = 0
h = 0

#scratch space
scratchvalue = ''

#holds temporary values
valuebuffer = ''



#Filename vars
#-------------
#filename of the model file given
filename = ''
#filename of the model file given without extension
filenamenoext = ''
#extension only of the model filename
filenameext = ''
#full path to the model file, plus the filename, but no extension
pathtomodelnoext = ''
#The directory where the files are with a '/' at the end
filedir = ''
#File handle of file currently being read
orfile = ''
#File handle of file currently being written
owfile = ''


NUM_VERTS = 0


OBJVertex = []
OBJNormal = []
OBJUV = []
OBJFaces = []

OBJNormalcollector = []

normalFaces = []
normalVerts = []


###########################################
#        Arguments                        #
###########################################
if len(sys.argv) < 2:
	print "Missing Argument:  An OBJ file"
	print
	raise SystemExit

filename = os.path.basename(sys.argv[1])
filedir = os.path.dirname(sys.argv[1])
if filedir != "":
	filedir += "/"
filenamenoext, filenameext = os.path.splitext(filename)
pathtomodelnoext = filedir+filenamenoext
if not os.path.isfile(filedir+filenamenoext+filenameext):
	print
	print "ERROR:File in argument does not exist"
	print
	raise SystemExit
################################################
################################################





###########################################
#        open CSK and create base OBJ     #
###########################################
try:
	orfile = open(pathtomodelnoext+".obj", 'rb')
except:
	print
	print "ERROR:Cannot open OBJ file for reading"
	print "Why? File seems to exist. Persmission problems?"
	print
	raise SystemExit


try:
	owfile = open(pathtomodelnoext+"-NormalLines.obj", 'w')
except:
	print
	print "ERROR:Cannot open file for writing."
	print "Disk Full? Directory read-only?"
	print
	orfile.close()
	raise SystemExit
################################################
################################################




###########################################
#            Read in OBJ                  #
###########################################
fileBuffer = ''
fileLength = 0


fileBuffer = orfile.readlines()
fileLength = len(fileBuffer)
###Find out how many verticies are in the OBJ's model
i = 0
while i < fileLength:
	valueBuffer = fileBuffer[i]
	valueBuffer = valueBuffer.strip()
	if valueBuffer == "":
		pass
	else:
		valueBuffer = valueBuffer.split()
		if valueBuffer[0] == " " or valueBuffer[0] == "\n":
			pass
		elif valueBuffer[0] == "v":
			NUM_VERTS += 1
		else:
			pass
	i += 1

###initialize the collector for the vertecies
i = 0
while i < NUM_VERTS:
	OBJNormalcollector.append( [] )
	i += 1


###Read in the OBJ
i = 0
while i < fileLength:
	valueBuffer = fileBuffer[i]
	valueBuffer = valueBuffer.strip()
	if valueBuffer == "":
		pass
	else:
		###Remember, when looking for the length of the split string, the leading letter
		###is still part of the string, if there should be three values, there will be
		###four entries after the split
		valueBuffer = valueBuffer.split()
		if valueBuffer[0] == " " or valueBuffer[0] == "\n":
			pass
		elif valueBuffer[0] == "v":
			if len(valueBuffer) != 4:
				print
				print "ERROR:OBJ formatting error"
				print "vertex number "+str(len(OBJVertex)+1)+" bad"
				print "Too many or too few values (3)?"
				print "Odd whitespace?"
				print
				owfile.close()
				orfile.close()
				raise SystemExit
			OBJVertex.append( ( float(valueBuffer[1]), float(valueBuffer[2]), float(valueBuffer[3]) ) )
		elif valueBuffer[0] == "vn":
			if len(valueBuffer) != 4:
				print
				print "ERROR:OBJ formatting error"
				print "normal number "+str(len(OBJNormal)+1)+" bad"
				print "Too many or too few values (3)?"
				print "Odd whitespace?"
				print
				owfile.close()
				orfile.close()
				raise SystemExit
			OBJNormal.append( ( float(valueBuffer[1]), float(valueBuffer[2]), float(valueBuffer[3]) ) )
		elif valueBuffer[0] == "vt":
			if len(valueBuffer) != 3:
				print
				print "ERROR:OBJ formatting error"
				print "UV number "+str(len(OBJUV)+1)+" bad"
				print "Too many or too few values (2)?"
				print "Odd whitespace?"
				print
				owfile.close()
				orfile.close()
				raise SystemExit
			OBJUV.append( ( float(valueBuffer[1]), float(valueBuffer[2]) ) )
		elif valueBuffer[0] == "f":
			if len(valueBuffer) != 4:
				print
				print "ERROR:OBJ formatting error"
				print "face number "+str(len(OBJFaces)+1)+" bad"
				print "Too many or too few vertices (3)?"
				print "Odd whitespace?"
				print "May only use trianlges currently"
				print
				owfile.close()
				orfile.close()
				raise SystemExit
			facesplit = valueBuffer[1].split("/") + valueBuffer[2].split("/") + valueBuffer[3].split("/")
			if len(facesplit) != 9:
				print
				print "ERROR:OBJ formatting error"
				print "face number "+str(len(OBJFaces)+1)+" bad"
				print "This script requires both normals and"
				print "UV points for each face vertex"
				print " ( format: #/#/# )"
				print
				owfile.close()
				orfile.close()
				raise SystemExit
			OBJFaces.append( ( facesplit[0] , facesplit[3] , facesplit[6] ) )
			OBJNormalcollector[int(facesplit[0])-1].append( int(facesplit[2])-1 )
			OBJNormalcollector[int(facesplit[3])-1].append( int(facesplit[5])-1 )
			OBJNormalcollector[int(facesplit[6])-1].append( int(facesplit[8])-1 )

		else:
			pass
	i += 1


orfile.close()


###########################################
#        Create new Face listing          #
###########################################
Nx = 0
Ny = 0
Nz = 0
vertexCount = 0
currentMainVert = 0

i = 0
while i < NUM_VERTS:
	normalVerts.append( ( OBJVertex[i][0], OBJVertex[i][1], OBJVertex[i][2] ) )
	currentMainVert = vertexCount
	vertexCount += 1
	filelength = len(OBJNormalcollector[i])
	j = 0
	while j < filelength:
		###The face index reference for this vertex's normal, used as the index into
		###the normal array to get the actual normal value
		Nx = OBJNormal[OBJNormalcollector[i][j]][0]
		Ny = OBJNormal[OBJNormalcollector[i][j]][1]
		Nz = OBJNormal[OBJNormalcollector[i][j]][2]
		normalVerts.append( ( OBJVertex[i][0]+Nx, OBJVertex[i][1]+Ny, OBJVertex[i][2]+Nz ) )
		###Add new faces (lines)
		normalFaces.append( ( currentMainVert , vertexCount ) )
		vertexCount += 1
		j += 1
	i += 1


###########################################
#             Write new OBJ               #
###########################################
filelength = len(normalVerts)
i = 0
while i < filelength:
	scratchvalue += "v " + str(normalVerts[i][0]) + " " + str(normalVerts[i][1]) + " " + str(normalVerts[i][2]) + "\n"
	i += 1


scratchvalue += "\n\n"

filelength = len(normalFaces)
i = 0
while i < filelength:
	###(+1) since OBJ indexes start from 1, not 0
	scratchvalue += "f " + str(normalFaces[i][0]+1) + "// " + str(normalFaces[i][1]+1) + "// \n"
	i += 1

owfile.write(scratchvalue)

owfile.close()




